
/* Copyright © 2012 Matthew Champion
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of mattunderscore.com nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL MATTHEW CHAMPION BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */

package com.mattunderscore.http.headers;

import java.util.Collection;

/**
 * Parser for the header field of an HTTP message.
 * 
 * @author Matt Champion
 * @param <T>
 *            The header field that is being parsed.
 * @since 0.0.13
 */
public interface HeaderParser<T extends HeaderFieldElement>
{

    /**
     * Test to see if the header field name can be parsed by this parser.
     * @param header Name of header field
     * @return Result of comparison
     * @since 0.0.13
     */
    public boolean isCorrectHeader(String header);

    /**
     * Parses a string for a list of header field elements.
     * @param header String of header field elements
     * @return The collection of header field elements
     * @throws UnParsableHeaderException
     * @since 0.0.13
     */
    public Collection<T> parseAll(String header)
            throws UnParsableHeaderException;

    /**
     * Parses a string for a single header field element.
     * @param header String of a header field element
     * @return The structured header field element
     * @throws UnParsableHeaderException
     * @since 0.0.13
     */
    public T parse(String header) throws UnParsableHeaderException;
}
